@@ -18,7 +18,7 @@ from surveydata.surveyplatform import SurveyPlatform
 from surveydata.storagesystem import StorageSystem
 import requests
 from urllib.parse import quote
-from datetime import datetime
+import datetime
 import pandas as pd
 
 
@@ -61,7 +61,7 @@ class SurveyCTOPlatform(SurveyPlatform):
         super().__init__()
 
     def sync_data(self, storage: StorageSystem, attachment_storage: StorageSystem = None,
-                  no_attachments: bool = False) -> list:
+                  no_attachments: bool = False, review_statuses: list = None) -> list:
         """
         Sync survey data to storage system.
 
@@ -71,6 +71,9 @@ class SurveyCTOPlatform(SurveyPlatform):
         :type attachment_storage: StorageSystem
         :param no_attachments: True to not sync attachments
         :type no_attachments: bool
+        :param review_statuses: List of review statuses to include (any combo of "approved", "pending", "rejected";
+            if not specified, syncs only approved submissions)
+        :type review_statuses: list
         :return: List of new submissions stored (submission ID strings)
         :rtype: list
         """
@@ -92,6 +95,10 @@ class SurveyCTOPlatform(SurveyPlatform):
         # pull data via server API
         api_url = f"https://{self.server}.surveycto.com/api/v2/forms/data/wide/json/{self.formid}?date=" \
                   + quote(cursor if cursor else "0")
+        # (with non-default list of review statuses, if supplied)
+        if review_statuses is not None and review_statuses:
+            api_url += "&r=" + quote("|".join(review_statuses))
+        # (and with private key, if supplied)
         if self.private_key:
             response = requests.post(api_url, files={"private_key": self.private_key}, auth=self.creds)
         else:
@@ -106,16 +113,16 @@ class SurveyCTOPlatform(SurveyPlatform):
         if data:
             # start out presuming that the last submission is the one with the latest CompletionDate
             newcursor = data[-1]["CompletionDate"]
-            newcursor_dt = datetime.strptime(newcursor, "%b %d, %Y %I:%M:%S %p")
+            newcursor_dt = datetime.datetime.strptime(newcursor, "%b %d, %Y %I:%M:%S %p")
 
             # loop through to process each submission
             for submission in data:
                 # if the submission's CompletionDate is greater than our presumed new cursor, use it instead
                 #   (this is out of an abundance of caution, as it appears that SurveyCTO sends submissions in
                 #    CompletionDate order)
-                if datetime.strptime(submission["CompletionDate"], "%b %d, %Y %I:%M:%S %p") > newcursor_dt:
+                if datetime.datetime.strptime(submission["CompletionDate"], "%b %d, %Y %I:%M:%S %p") > newcursor_dt:
                     newcursor = submission["CompletionDate"]
-                    newcursor_dt = datetime.strptime(newcursor, "%b %d, %Y %I:%M:%S %p")
+                    newcursor_dt = datetime.datetime.strptime(newcursor, "%b %d, %Y %I:%M:%S %p")
 
                 # generally, we want to write submissions to storage, even if we already have them — but, for
                 # efficiency reasons, we don't want to keep re-storing the most recent submission when it matches
@@ -155,6 +162,9 @@ class SurveyCTOPlatform(SurveyPlatform):
             if newcursor != cursor:
                 storage.store_metadata(self.CURSOR_METADATA_ID, newcursor)
 
+                # since the cursor changed, go ahead an set the appropriate (API) timezone as well
+                storage.set_data_timezone(datetime.timezone.utc)
+
         return new_submission_list
 
     @staticmethod
@@ -206,7 +216,7 @@ class SurveyCTOPlatform(SurveyPlatform):
         elif location_strings is not None:
             df = None
             for subid, locstr in location_strings.items():
-                if locstr:
+                if not pd.isna(locstr) and locstr:
                     row_df = SurveyCTOPlatform._load_text_audit(storage, locstr)
                     row_df[SurveyCTOPlatform.ID_FIELD] = subid
                     if df is None:
@@ -215,11 +225,30 @@ class SurveyCTOPlatform(SurveyPlatform):
                         df = pd.concat([df, row_df], copy=False, ignore_index=True, sort=False)
 
             # set to index by KEY
-            df.set_index([SurveyCTOPlatform.ID_FIELD], inplace=True)
-            df = df.sort_index()
+            if df is not None:
+                df.set_index([SurveyCTOPlatform.ID_FIELD], inplace=True)
+                df = df.sort_index()
         else:
             raise ValueError("Pass either location_string or location_strings to load one or more text audits.")
 
+        # before returning, optimize column data types
+        if df is not None:
+            # parse device_time column, if present
+            dt_col = "device_time"
+            if dt_col in df.columns.values:
+                # count our non-NaN, non-empty-string values
+                nvals = df.loc[df[dt_col] != "", dt_col].count()
+
+                if nvals > 0:
+                    # try converting to datetime
+                    converted = pd.to_datetime(df[dt_col], errors="coerce")
+                    # if we didn't lose any data in the process, go with the converted version
+                    if converted.count() == nvals:
+                        df[dt_col] = converted
+
+            # convert data types based on object types
+            df = df.convert_dtypes()
+
         return df
 
     @staticmethod
